
# install.packages("rpart")
# install.packages("randomForest")
# install.packages("RWeka")

library(rpart)
library(randomForest)
library(RWeka)

DNA <- read.csv("splice_data.txt", header=T)

l <- length(DNA[,1])
ls <- sample(1:l,2*l/3)
DNA.ls <- DNA[ls,]
DNA.ts <- DNA[-ls,]

lls <- dim(DNA.ls)[1]
s1 <- sample(1:lls,2*lls/3)
DNA.ls.s1 <- DNA.ls[s1,]
DNA.ls.s2 <- DNA.ls[-s1,]

# build a fully grown tree
model.dt <- rpart(Class ~ .,method="class", data=DNA.ls.s1)
# prune it using 10-fold cross-validation
model.pruneddt <- prune(model.dt, cp=model.dt$cptable[which.min(model.dt$cptable[,"xerror"]),"CP"])
# compute predictions on S2
pred.pruneddt <- predict(model.pruneddt,newdata=DNA.ls.s2,type="class")
# estimate accuracy
acc.pruneddt <- 100*sum(pred.pruneddt==DNA.ls.s2$Class)/dim(DNA.ls.s2)[1]
acc.pruneddt

# Build a random forest model
model.rf <- randomForest(Class~.,ntreeTry=500,data=DNA.ls.s1)
# compute predictions on s2
pred.rf <- predict(model.rf,newdata=DNA.ls.s2,type="class")
# estimate accuracy
acc.rf <- 100*sum(pred.rf==DNA.ls.s2$Class)/dim(DNA.ls.s2)[1]
acc.rf

# tune the parameter K using out-of-bag estimates
model.rftuned <- tuneRF(DNA.ls.s1[,1:60],DNA.ls.s1$Class,ntreeTry=500,improve=0.001,stepFactor=1.5,plot=TRUE,doBest=TRUE)
pred.rftuned <- predict(model.rftuned,newdata=DNA.ls.s2,type="class")
# estimate accuracy
acc.rftuned <- 100*sum(pred.rftuned==DNA.ls.s2$Class)/dim(DNA.ls.s2)[1]
acc.rftuned

# AdaBoost
# List of values of M to consider (from smaller to larger trees)
lm <- c(200,100,50,30,20,15,10,1)
acc <- NA
# cycle through all values
for (i in 1:length(lm)) {
  m.adaboost <- AdaBoostM1(Class~., data=DNA.ls.s1, control=Weka_control(I=500,W=list(J48,M=lm[i],U=TRUE)))
  eval <- evaluate_Weka_classifier(m.adaboost,numFolds=5,seed=123)
  acc[i] <- eval$details[1]
}
# plot evolution of the 5-fold CV accuracy with respect to M
plot(lm,acc,type="l",xlab="M",ylab="Accuracy (%)")
# build a new model with the optimal setting
optm <- lm[which.max(acc)]
model.adaboost <- AdaBoostM1(Class~., data=DNA.ls.s1, control=Weka_control(I=500,W=list(J48,M=optm,U=TRUE)))
# Evaluate its accuracy
pred.adaboost <- predict(model.adaboost,newdata=DNA.ls.s2,type="class")
acc.adaboost <- 100*sum(pred.adaboost==DNA.ls.s2$Class)/dim(DNA.ls.s2)[1]
acc.adaboost

# Build the final model on the whole learning sample
finalmodel <- randomForest(Class~.,ntreeTry=500,data=DNA.ls,mtry=10)
finalpred <- predict(finalmodel,newdata=DNA.ts,type="class")
# Evaluate its accuracy
finalacc <- 100*sum(finalpred==DNA.ts$Class)/dim(DNA.ts)[1]
finalacc

# grow and prune the decision tree
model.dt <- rpart(Class ~ .,method="class", data=DNA.ls)
model.pruneddt <- prune(model.dt, cp=model.dt$cptable[which.min(model.dt$cptable[,"xerror"]),"CP"])
# plot it
par(mfrow=c(1,1), xpd=NA)
plot(model.pruneddt, uniform=TRUE, main="Pruned Tree for DNA", cex.main=1.5)
text(model.pruneddt, use.n=TRUE, cex=.8, pretty=0)

# grow a random forest model with variable importance computation
model.rf <- randomForest(Class~.,ntreeTry=500,data=DNA.ls,importance=TRUE,mtry=10)
# plot variable importances (without sorting)
varImpPlot(model.rf,type=1,sort=FALSE,n.var=60,main="Variable Importance for DNA", cex.main=1.5, cex.lab=1.5)
